# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects.disk_object import Disk
from cybox.test.objects import ObjectTestCase


class TestDisk(ObjectTestCase, unittest.TestCase):
    object_type = "DiskObjectType"
    klass = Disk

    _full_dict = {
        'disk_name': u"A disk",
        'disk_size': 12345678,
        'free_space': 1234567,
        'partition_list': [
            {
                'device_name': "A partition",
                'xsi:type': "DiskPartitionObjectType",
            },
            {
                'device_name': "B partition",
                'xsi:type': "DiskPartitionObjectType",
            },
        ],
        'type': "Fixed",
        'xsi:type': object_type,
    }

    # https://github.com/CybOXProject/python-cybox/issues/267
    def test_type(self):
        disk = Disk()
        disk.type_ = "Fixed"
        self.assertTrue(b"Fixed" in disk.to_xml())


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox.vendor import six

from .version import __version__  # noqa


#TODO: Should this get moved to mixbox or not?
class Unicode(entities.Entity):
    """Shim class to allow xs:string's in EntityList"""

    def __init__(self, value):
        super(Unicode, self).__init__()
        self.value = value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = six.text_type(value)

    def to_obj(self, ns_info=None):
        return self.value

    def to_dict(self):
        return self.value

    @classmethod
    def from_obj(cls, cls_obj):
        return cls(cls_obj)

    from_dict = from_obj



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields
from mixbox.vendor import six
from mixbox.vendor.six import u

import cybox.bindings.cybox_common as common_binding
from cybox.common import HexBinary, String, VocabString
from cybox.common.vocabs import HashName, VocabField


def _set_hash_type(entity, value):
    """Callback hook to set the hash type based on the length of the value.

    If the `Hash` object already has a type, it is not changed.

    Args:
        entity (Hash): The Hash object being modified.
        value (str): The hash value
    """
    # If the Hash already has a defined type_, exit early:
    if entity.type_:
        return
    if not value:
        return
    # The `value` argument should be a HexBinary object, so we look at the
    # string in its `value` field.
    hashlen = len(value.value)
    if hashlen == 32:
        entity.type_ = Hash.TYPE_MD5
    elif hashlen == 40:
        entity.type_ = Hash.TYPE_SHA1
    elif hashlen == 56:
        entity.type_ = Hash.TYPE_SHA224
    elif hashlen == 64:
        entity.type_ = Hash.TYPE_SHA256
    elif hashlen == 96:
        entity.type_ = Hash.TYPE_SHA384
    elif hashlen == 128:
        entity.type_ = Hash.TYPE_SHA512
    else:
        entity.type_ = Hash.TYPE_OTHER


class Hash(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.HashType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'

    type_ = VocabField("Type", HashName)
    simple_hash_value = fields.TypedField("Simple_Hash_Value", HexBinary,
                                          postset_hook=_set_hash_type)
    fuzzy_hash_value = fields.TypedField("Fuzzy_Hash_Value", String)

    TYPE_MD5 = u("MD5")
    TYPE_MD6 = u("MD6")
    TYPE_SHA1 = u("SHA1")
    TYPE_SHA224 = u("SHA224")
    TYPE_SHA256 = u("SHA256")
    TYPE_SHA384 = u("SHA384")
    TYPE_SHA512 = u("SHA512")
    TYPE_SSDEEP = u("SSDEEP")
    TYPE_OTHER = VocabString(u("Other"))

    def __init__(self, hash_value=None, type_=None, exact=False):
        """Create a new Hash Object

        If exact=True, add 'condition="Equals"' to the hash value and type.
        """
        super(Hash, self).__init__()
        # Set type_ first so that auto-typing will work.
        self.type_ = type_
        self.simple_hash_value = hash_value

        if exact:
            if self.simple_hash_value:
                self.simple_hash_value.condition = "Equals"
            if self.type_:
                self.type_.condition = "Equals"

    def __str__(self):
        return str(self.simple_hash_value)

    # Other_Type and FuzzyHashes not yet supported.

#    @classmethod
#    def object_from_dict(cls, hash_dict):
#        """Create the Hash object representation from an input dictionary"""
#        hash = common_binding.HashType()
#        for hash_key, hash_value in hash_dict.items():
#            if hash_key == 'type' : hash.Type = Base_Object_Attribute.object_from_dict(common_binding.StringObjectAttributeType(datatype='String'),hash_value)
#            if hash_key == 'other_type' : hash.Type = Base_Object_Attribute.object_from_dict(common_binding.StringObjectAttributeType(datatype='String'),hash_value)
#            if hash_key == 'simple_hash_value' : hash.Simple_Hash_Value = Base_Object_Attribute.object_from_dict(common_binding.HexBinaryObjectAttributeType(datatype='hexBinary'),hash_value)
#            if hash_key == 'fuzzy_hash_value' : hash.Fuzzy_Hash_Value = Base_Object_Attribute.object_from_dict(common_binding.StringObjectAttributeType(datatype='String'),hash_value)
#            if hash_key == 'fuzzy_hash_structure':
#                for fuzzy_hash_structure_dict in hash_value:
#                    fuzzy_hash_structure_object = common_binding.FuzzyHashStructureType()
#                    for fuzzy_key, fuzzy_value in fuzzy_hash_structure_dict.items():
#                        if fuzzy_key == 'block_size' : fuzzy_hash_structure_object.Block_Size = Base_Object_Attribute.object_from_dict(common_binding.IntegerObjectAttributeType(datatype='Integer'),fuzzy_value)
#                        if fuzzy_key == 'block_hash' :
#                            block_hash_dict = fuzzy_value
#                            block_hash_object = common_binding.FuzzyHashBlockType()
#                            for block_hash_key, block_hash_value in block_hash_dict.items():
#                                if block_hash_key == 'segment_count' : block_hash_object.Segment_Count = Base_Object_Attribute.object_from_dict(common_binding.IntegerObjectAttributeType(datatype='Integer'),block_hash_value)
#                                if block_hash_key == 'block_hash_value' :
#                                    hash_value_dict = block_hash_value
#                                    hash_value_object = common_binding.HashValueType()
#                                    for hash_value_key, hash_value_value in hash_value_dict.items():
#                                        if hash_value_key == 'simple_hash_value' : hash_value_object.Simple_Hash_Value = Base_Object_Attribute.object_from_dict(common_binding.HexBinaryObjectAttributeType(datatype='hexBinary'),hash_value_value)
#                                        if hash_value_key == 'fuzzy_hash_value' : hash_value_object.Fuzzy_Hash_Value = Base_Object_Attribute.object_from_dict(common_binding.StringObjectAttributeType(datatype='String'),hash_value_value)
#                                    if hash_value_object.hasContent_(): block_hash_object.Block_Hash_Value = hash_value_object
#                                if block_hash_key == 'segments' :
#                                    segments_dict = block_hash_value
#                                    segments_object = common_binding.HashSegmentsType()
#                                    for segment in segments_dict:
#                                        hash_segment_object = common_binding.HashSegmentType()
#                                        for segment_key, segment_value in segment.items():
#                                            if segment_key == 'trigger_point' : hash_segment_object.Trigger_Point = Base_Object_Attribute.object_from_dict(common_binding.HexBinaryObjectAttributeType(datatype='hexBinary'),segment_value)
#                                            if segment_key == 'segment_hash' :
#                                                segment_hash_dict = segment_value
#                                                segment_hash_object = common_binding.HashValueType()
#                                                for segment_hash_key, segment_hash_value in segment_hash_dict.items():
#                                                    if segment_hash_key == 'simple_hash_value' : segment_hash_object.Simple_Hash_Value = Base_Object_Attribute.object_from_dict(common_binding.HexBinaryObjectAttributeType(datatype='hexBinary'),segment_hash_value)
#                                                    if segment_hash_key == 'fuzzy_hash_value' : segment_hash_object.Fuzzy_Hash_Value = Base_Object_Attribute.object_from_dict(common_binding.StringObjectAttributeType(datatype='String'),segment_hash_value)
#                                                if segment_hash_object.hasContent_(): hash_segment_object.Segment_Hash = segment_hash_object
#                                            if segment_key == 'raw_segment_content' : hash_segment_object.Raw_Segment_Content = segment_value
#                                        if hash_segment_object.hasContent_() : segments_object.add_Segment(hash_segment_object)
#                                    if segments_object.hasContent_() : block_hash_object.Segments = segments_object
#                            if block_hash_object.hasContent_() : fuzzy_hash_structure_object.Block_Hash = block_hash_object
#                    if fuzzy_hash_structure_object.hasContent_() : hash.add_Fuzzy_Hash_Structure(fuzzy_hash_structure_object)
#
#        return hash


class HashList(entities.EntityList):
    _binding = common_binding
    _binding_class = common_binding.HashListType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'
    hashes = fields.TypedField("Hash", Hash, multiple=True)

    @property
    def md5(self):
        return self._get_hash_value(Hash.TYPE_MD5)

    @md5.setter
    def md5(self, value):
        self._set_hash(Hash.TYPE_MD5, value)

    @property
    def sha1(self):
        return self._get_hash_value(Hash.TYPE_SHA1)

    @sha1.setter
    def sha1(self, value):
        self._set_hash(Hash.TYPE_SHA1, value)

    @property
    def sha224(self):
        return self._get_hash_value(Hash.TYPE_SHA224)

    @sha224.setter
    def sha224(self, value):
        self._set_hash(Hash.TYPE_SHA224, value)

    @property
    def sha256(self):
        return self._get_hash_value(Hash.TYPE_SHA256)

    @sha256.setter
    def sha256(self, value):
        self._set_hash(Hash.TYPE_SHA256, value)

    @property
    def sha384(self):
        return self._get_hash_value(Hash.TYPE_SHA384)

    @sha384.setter
    def sha384(self, value):
        self._set_hash(Hash.TYPE_SHA384, value)

    @property
    def sha512(self):
        return self._get_hash_value(Hash.TYPE_SHA512)

    @sha512.setter
    def sha512(self, value):
        self._set_hash(Hash.TYPE_SHA512, value)

    def _hash_lookup(self, type_):
        for h in self:
            if h.type_ == type_:
                return h.simple_hash_value
        return None

    def _set_hash(self, type_, value):
        h = self._hash_lookup(type_)
        if h:
            h.simple_hash_value = value
        else:
            self.append(Hash(value, type_))

    def _get_hash_value(self, type_):
        """Return the hash with a given type_, or None"""
        h = self._hash_lookup(type_)
        if h:
            return h.value
        return None



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""CybOX Action Reference Class"""

from mixbox import entities
from mixbox import fields

import cybox.bindings.cybox_core as core_binding


class ActionReference(entities.Entity):
    _binding = core_binding
    _binding_class = core_binding.ActionReferenceType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'

    action_id = fields.TypedField("action_id")

    def __init__(self, action_id=None):
        super(ActionReference, self).__init__()
        self.action_id = action_id



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects import _ObjectMetadata, UnknownObjectType


class ObjectTypeTest(unittest.TestCase):

    def setUp(self):
        obj_list = [
            ("!!MissingAPIClass", None, None, None, None),
            ("!!MissingModule", 'some.nonexistent.module', None, None, None),
            ("!!BadClassName", 'cybox.NonexistentClass', None, None, None),
        ]
        self.meta = _ObjectMetadata(obj_list)

    def test_unknown_object(self):
        self.assertRaises(UnknownObjectType,
                          self.meta.get_class_for_object_type,
                          "!!BadType")

    def test_missing_api_class(self):
        self.assertRaises(UnknownObjectType,
                          self.meta.get_class_for_object_type,
                          "!!MissingAPIClass")

    def test_missing_module(self):
        self.assertRaises(ImportError,
                          self.meta.get_class_for_object_type,
                          "!!MissingModule")

    def test_missing_class(self):
        self.assertRaises(AttributeError,
                          self.meta.get_class_for_object_type,
                          "!!BadClassName")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""Creates the CybOX content for CybOX_Artifact_Instance.xml
"""

import os

from cybox.core import Observable, Observables
from cybox.objects.artifact_object import Artifact, Base64Encoding


def main():
    test_file = os.path.join(os.path.dirname(__file__), "test.pcap")

    with open(test_file, "rb") as f:
        data = f.read()

    a = Artifact(data, Artifact.TYPE_NETWORK)
    a.packaging.append(Base64Encoding())

    o = Observable(a)

    o.description = ("This Observable specifies an instance of an Artifact "
                     "object, specifically some network traffic that was "
                     "captured in a PCAP file and then base64 encoded for "
                     "transport.")

    print(Observables(o).to_xml(encoding=None))


if __name__ == "__main__":
    main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.pdf_file_object as pdf_file_binding
from cybox.objects.file_object import File
from cybox.common import DateTime, PositiveInteger, String, NonNegativeInteger, Double


class PDFKeywordCount(entities.Entity):
    _binding = pdf_file_binding
    _binding_class = pdf_file_binding.PDFKeywordCountType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/pdf-file-1'

    non_obfuscated_count = fields.TypedField("Non_Obfuscated_Count", NonNegativeInteger)
    obfuscated_count = fields.TypedField("Obfuscated_Count", NonNegativeInteger)


class PDFKeywordCounts(entities.Entity):
    _binding = pdf_file_binding
    _binding_class = pdf_file_binding.PDFKeywordCountsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/pdf-file-1'

    page_count = fields.TypedField("Page_Count", PDFKeywordCount)
    encrypt_count = fields.TypedField("Encrypt_Count", PDFKeywordCount)
    objstm_count = fields.TypedField("ObjStm_Count", PDFKeywordCount)
    js_count = fields.TypedField("JS_Count", PDFKeywordCount)
    javascript_count = fields.TypedField("JavaScript_Count", PDFKeywordCount)
    aa_count = fields.TypedField("AA_Count", PDFKeywordCount)
    openaction_count = fields.TypedField("OpenAction_Count", PDFKeywordCount)
    asciihexdecode_count = fields.TypedField("ASCIIHexDecode_Count", PDFKeywordCount)
    ascii85decode_count = fields.TypedField("ASCII85Decode_Count", PDFKeywordCount)
    lzwdecode_count = fields.TypedField("LZWDecode_Count", PDFKeywordCount)
    flatedecode_count = fields.TypedField("FlateDecode_Count", PDFKeywordCount)
    runlengthdecode_count = fields.TypedField("RunLengthDecode_Count", PDFKeywordCount)
    jbig2decode_count = fields.TypedField("JBIG2Decode_Count", PDFKeywordCount)
    dctdecode_count = fields.TypedField("DCTDecode_Count", PDFKeywordCount)
    richmedia_count = fields.TypedField("RichMedia_Count", PDFKeywordCount)
    ccittfaxdecode_count = fields.TypedField("CCITTFaxDecode_Count", PDFKeywordCount)
    launch_count = fields.TypedField("Launch_Count", PDFKeywordCount)
    xfa_count = fields.TypedField("XFA_Count", PDFKeywordCount)


class PDFDocumentInformationDictionary(entities.Entity):
    _binding = pdf_file_binding
    _binding_class = pdf_file_binding.PDFDocumentInformationDictionaryType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/pdf-file-1'

    title = fields.TypedField("Title", String)
    author = fields.TypedField("Author", String)
    subject = fields.TypedField("Subject", String)
    keywords = fields.TypedField("Keywords", String)
    creator = fields.TypedField("Creator", String)
    producer = fields.TypedField("Producer", String)
    creationdate = fields.TypedField("CreationDate", DateTime)
    moddate = fields.TypedField("ModDate", DateTime)
    trapped = fields.TypedField("Trapped", String)


class PDFFileMetadata(entities.Entity):
    _binding = pdf_file_binding
    _binding_class = pdf_file_binding.PDFFileMetadataType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/pdf-file-1'

    encrypted = fields.TypedField("encrypted")
    optimized = fields.TypedField("optimized")
    document_information_dictionary = fields.TypedField("Document_Information_Dictionary", PDFDocumentInformationDictionary)
    number_of_indirect_objects = fields.TypedField("Number_Of_Indirect_Objects", PositiveInteger)
    number_of_trailers = fields.TypedField("Number_Of_Trailers", PositiveInteger)
    number_of_cross_reference_tables = fields.TypedField("Number_Of_Cross_Reference_Tables", PositiveInteger)
    keyword_counts = fields.TypedField("Keyword_Counts", PDFKeywordCounts)


class PDFFile(File):
    _binding = pdf_file_binding
    _binding_class = pdf_file_binding.PDFFileObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/pdf-file-1'
    _XSI_NS = "PDFFileObj"
    _XSI_TYPE = "PDFFileObjectType"

    metadata = fields.TypedField("Metadata", PDFFileMetadata)
    version = fields.TypedField("Version", Double)
    #TODO: implement remaining elements
    #indirect_objects = fields.TypedField("Indirect_Objects", PDFIndirectObjectList)
    #cross_reference_tables = fields.TypedField("Cross_Reference_Tables", PDFXrefTableList)
    #trailers = fields.TypedField("Trailers", PDFTrailerList)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.socket_address_object as socket_address_binding
from cybox.objects.address_object import Address
from cybox.objects.port_object import Port
from cybox.objects.hostname_object import Hostname
from cybox.common import ObjectProperties


class SocketAddress(ObjectProperties):
    _binding = socket_address_binding
    _binding_class = socket_address_binding.SocketAddressObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/socket-address-1"
    _XSI_NS = "SocketAddressObj"
    _XSI_TYPE = "SocketAddressObjectType"

    #TODO: make sure this is an IPV4 or IPV6 Address
    #TODO: choice between ip_address and hostname
    ip_address = fields.TypedField("IP_Address", Address)
    hostname = fields.TypedField("Hostname", Hostname)
    port = fields.TypedField("Port", Port)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# TODO: This module should probably move to mixbox.
import functools

from mixbox import entities
from mixbox import fields
from mixbox import typedlist
from mixbox.vendor import six

import cybox.bindings.cybox_common as common_binding
from cybox.common import PatternFieldGroup
from cybox.utils import normalize_to_xml, denormalize_from_xml


def validate_value(instance, value):
    allowed = instance._ALLOWED_VALUES

    if not value:
        return
    elif not allowed:
        return
    elif value in allowed:
        return
    else:
        error = "Value for vocab {instance.__class__} must be one of {allowed}. Received '{value}'"
        error = error.format(**locals())
        raise ValueError(error)


class VocabFactory(entities.EntityFactory):
    _convert_strings = True
    
    @classmethod
    def entity_class(cls, key):
        if not key:
            return VocabString

        for xsitype, klass in six.iteritems(_VOCAB_MAP):
            if key in xsitype:
                return klass

        return VocabString


class VocabList(typedlist.TypedList):
    """VocabString fields can be any type of VocabString, though there is often
    a preferred/default VocabString type.

    The TypedList will attempt to make sure that every input item is an instance
    of the default VocabString and throw an error if it isn't. This sublcass
    overrides that behavior and allows any instance of VocabString to be
    inserted.
    """

    def _is_valid(self, value):
        return isinstance(value, VocabString)


class VocabField(fields.TypedField):
    """TypedField subclass for VocabString fields."""

    def __init__(self, *args, **kwargs):
        """Intercepts the __init__() call to TypedField.

        Set the type that will be used in from_dict and from_obj calls to
        :class:`VocabString`.

        Set the type that will be used in ``__set__`` for casting as the
        original ``type_`` argument, or :class:`VocabString` if no `type_`
        argument was provided.

        """
        super(VocabField, self).__init__(*args, **kwargs)
        self.factory = VocabFactory  # force this factory

        if self._unresolved_type is None:
            self.type_ = VocabString

        self._listfunc = functools.partial(VocabList, type=self._unresolved_type)

    def check_type(self, value):
        return isinstance(value, VocabString)


class VocabString(PatternFieldGroup, entities.Entity):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    # All subclasses should override this
    _XSI_TYPE = None
    _ALLOWED_VALUES = ()
    _binding = common_binding
    _binding_class = common_binding.ControlledVocabularyStringType

    value = fields.TypedField("valueOf_", key_name="value", preset_hook=validate_value)
    vocab_name = fields.TypedField("vocab_name")
    vocab_reference = fields.TypedField("vocab_reference")
    xsi_type = fields.TypedField("xsi_type", key_name="xsi:type")

    def __init__(self, value=None):
        super(VocabString, self).__init__()
        self.value = value
        self.xsi_type = self._XSI_TYPE
        self.vocab_name = None
        self.vocab_reference = None

    def __str__(self):
        return str(self.value)

    def __eq__(self, other):
        # Check to make sure the values are identical.
        if isinstance(other, VocabString):
            other = other.value
        return other == self.value

    def is_plain(self):
        """Whether the VocabString can be represented as a single value.

        """        
        return (
            (self.xsi_type is None or type(self)._XSI_TYPE == self.xsi_type) and
            self.vocab_name is None and
            self.vocab_reference is None and
            super(VocabString, self).is_plain()
        )

    def to_obj(self, ns_info=None):
        obj = super(VocabString, self).to_obj(ns_info=ns_info)
        obj.valueOf_ = normalize_to_xml(self.value, self.delimiter)
        return obj

    def to_dict(self):
        if self.is_plain():
            return self.value
        return super(VocabString, self).to_dict()

    @classmethod
    def from_obj(cls, cls_obj):
        if not cls_obj:
            return None

        obj = super(VocabString, cls).from_obj(cls_obj)
        obj.value = denormalize_from_xml(value=cls_obj.valueOf_, delimiter=obj.delimiter)
        return obj


#: Mapping of Controlled Vocabulary xsi:type's to their class implementations.
_VOCAB_MAP = {}

def _get_terms(vocab_class):
    """Helper function used by register_vocab."""
    for k, v in vocab_class.__dict__.items():
        if k.startswith("TERM_"):
            yield v


def register_vocab(cls):
    """Register a VocabString subclass.

    Also, calculate all the permitted values for class being decorated by
    adding an ``_ALLOWED_VALUES`` tuple of all the values of class members
    beginning with ``TERM_``.
    """
    _VOCAB_MAP[cls._XSI_TYPE] = cls  # noqa

    cls._ALLOWED_VALUES = tuple(_get_terms(cls))
    return cls


@register_vocab
class EventType(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:EventTypeVocab-1.0.1'
    _VOCAB_VERSION = '1.0.1'

    TERM_ACCOUNT_OPS_APP_LAYER = 'Account Ops (App Layer)'
    TERM_ANOMALY_EVENTS = 'Anomaly Events'
    TERM_API_CALLS = 'API Calls'
    TERM_APPLICATION_LOGIC = 'Application Logic'
    TERM_APP_LAYER_TRAFFIC = 'App Layer Traffic'
    TERM_AUTHENTICATION_OPS = 'Authentication Ops'
    TERM_AUTHORIZATION_ACL = 'Authorization (ACL)'
    TERM_AUTORUN = 'Autorun'
    TERM_AUTO_UPDATE_OPS = 'Auto-update Ops'
    TERM_BASIC_SYSTEM_OPS = 'Basic System Ops'
    TERM_CONFIGURATION_MANAGEMENT = 'Configuration Management'
    TERM_DATA_FLOW = 'Data Flow'
    TERM_DHCP = 'DHCP'
    TERM_DNS_LOOKUP_OPS = 'DNS Lookup Ops'
    TERM_EMAIL_OPS = 'Email Ops'
    TERM_FILE_OPS_CRUD = 'File Ops (CRUD)'
    TERM_GUI_KVM = 'GUI/KVM'
    TERM_HTTP_TRAFFIC = 'HTTP Traffic'
    TERM_IPC = 'IPC'
    TERM_IP_OPS = 'IP Ops'
    TERM_MEMORY_OPS = 'Memory Ops'
    TERM_PACKET_TRAFFIC = 'Packet Traffic'
    TERM_PORT_SCAN = 'Port Scan'
    TERM_PRIVILEGE_OPS = 'Privilege Ops'
    TERM_PROCEDURAL_COMPLIANCE = 'Procedural Compliance'
    TERM_PROCESS_MGT = 'Process Mgt'
    TERM_REDIRECTION = 'Redirection'
    TERM_REGISTRY_OPS = 'Registry Ops'
    TERM_SERVICE_MGT = 'Service Mgt'
    TERM_SESSION_MGT = 'Session Mgt'
    TERM_SIGNATURE_DETECTION = 'Signature Detection'
    TERM_SOCKET_OPS = 'Socket Ops'
    TERM_SQL = 'SQL'
    TERM_TECHNICAL_COMPLIANCE = 'Technical Compliance'
    TERM_THREAD_MGT = 'Thread Mgt'
    TERM_USB_MEDIA_DETECTION = 'USB/Media Detection'
    TERM_USER_PASSWORD_MGT = 'User/Password Mgt'


@register_vocab
class ActionType(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:ActionTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ACCEPT = 'Accept'
    TERM_ACCESS = 'Access'
    TERM_ADD = 'Add'
    TERM_ALERT = 'Alert'
    TERM_ALLOCATE = 'Allocate'
    TERM_ARCHIVE = 'Archive'
    TERM_ASSIGN = 'Assign'
    TERM_AUDIT = 'Audit'
    TERM_BACKUP = 'Backup'
    TERM_BIND = 'Bind'
    TERM_BLOCK = 'Block'
    TERM_CALL = 'Call'
    TERM_CHANGE = 'Change'
    TERM_CHECK = 'Check'
    TERM_CLEAN = 'Clean'
    TERM_CLICK = 'Click'
    TERM_CLOSE = 'Close'
    TERM_COMPARE = 'Compare'
    TERM_COMPRESS = 'Compress'
    TERM_CONFIGURE = 'Configure'
    TERM_CONNECT = 'Connect'
    TERM_CONTROL = 'Control'
    TERM_COPY_DUPLICATE = 'Copy/Duplicate'
    TERM_CREATE = 'Create'
    TERM_DECODE = 'Decode'
    TERM_DECOMPRESS = 'Decompress'
    TERM_DECRYPT = 'Decrypt'
    TERM_DENY = 'Deny'
    TERM_DEPRESS = 'Depress'
    TERM_DETECT = 'Detect'
    TERM_DISCONNECT = 'Disconnect'
    TERM_DOWNLOAD = 'Download'
    TERM_DRAW = 'Draw'
    TERM_DROP = 'Drop'
    TERM_ENCODE = 'Encode'
    TERM_ENCRYPT = 'Encrypt'
    TERM_ENUMERATE = 'Enumerate'
    TERM_EXECUTE = 'Execute'
    TERM_EXTRACT = 'Extract'
    TERM_FILTER = 'Filter'
    TERM_FIND = 'Find'
    TERM_FLUSH = 'Flush'
    TERM_FORK = 'Fork'
    TERM_FREE = 'Free'
    TERM_GET = 'Get'
    TERM_HIDE = 'Hide'
    TERM_HOOK = 'Hook'
    TERM_IMPERSONATE = 'Impersonate'
    TERM_INITIALIZE = 'Initialize'
    TERM_INJECT = 'Inject'
    TERM_INSTALL = 'Install'
    TERM_INTERLEAVE = 'Interleave'
    TERM_JOIN = 'Join'
    TERM_KILL = 'Kill'
    TERM_LISTEN = 'Listen'
    TERM_LOAD = 'Load'
    TERM_LOCK = 'Lock'
    TERM_LOGIN_LOGON = 'Login/Logon'
    TERM_LOGOUT_LOGOFF = 'Logout/Logoff'
    TERM_MAP = 'Map'
    TERM_MERGE = 'Merge'
    TERM_MODIFY = 'Modify'
    TERM_MONITOR = 'Monitor'
    TERM_MOVE = 'Move'
    TERM_OPEN = 'Open'
    TERM_PACK = 'Pack'
    TERM_PAUSE = 'Pause'
    TERM_PRESS = 'Press'
    TERM_PROTECT = 'Protect'
    TERM_QUARANTINE = 'Quarantine'
    TERM_QUERY = 'Query'
    TERM_QUEUE = 'Queue'
    TERM_RAISE = 'Raise'
    TERM_READ = 'Read'
    TERM_RECEIVE = 'Receive'
    TERM_RELEASE = 'Release'
    TERM_REMOVE_DELETE = 'Remove/Delete'
    TERM_RENAME = 'Rename'
    TERM_REPLICATE = 'Replicate'
    TERM_RESTORE = 'Restore'
    TERM_RESUME = 'Resume'
    TERM_REVERT = 'Revert'
    TERM_RUN = 'Run'
    TERM_SAVE = 'Save'
    TERM_SCAN = 'Scan'
    TERM_SCHEDULE = 'Schedule'
    TERM_SEARCH = 'Search'
    TERM_SEND = 'Send'
    TERM_SET = 'Set'
    TERM_SHUTDOWN = 'Shutdown'
    TERM_SLEEP = 'Sleep'
    TERM_SNAPSHOT = 'Snapshot'
    TERM_START = 'Start'
    TERM_STOP = 'Stop'
    TERM_SUSPEND = 'Suspend'
    TERM_SYNCHRONIZE = 'Synchronize'
    TERM_THROW = 'Throw'
    TERM_TRANSMIT = 'Transmit'
    TERM_UNBLOCK = 'Unblock'
    TERM_UNHIDE = 'Unhide'
    TERM_UNHOOK = 'Unhook'
    TERM_UNINSTALL = 'Uninstall'
    TERM_UNLOAD = 'Unload'
    TERM_UNLOCK = 'Unlock'
    TERM_UNMAP = 'Unmap'
    TERM_UNPACK = 'Unpack'
    TERM_UPDATE = 'Update'
    TERM_UPGRADE = 'Upgrade'
    TERM_UPLOAD = 'Upload'
    TERM_WIPE_DESTROY_PURGE = 'Wipe/Destroy/Purge'
    TERM_WRITE = 'Write'


@register_vocab
class ActionObjectAssociationType(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:ActionObjectAssociationTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_AFFECTED = 'Affected'
    TERM_INITIATING = 'Initiating'
    TERM_RETURNED = 'Returned'
    TERM_UTILIZED = 'Utilized'


@register_vocab
class HashName(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:HashNameVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_MD5 = 'MD5'
    TERM_MD6 = 'MD6'
    TERM_SHA1 = 'SHA1'
    TERM_SHA224 = 'SHA224'
    TERM_SHA256 = 'SHA256'
    TERM_SHA384 = 'SHA384'
    TERM_SHA512 = 'SHA512'
    TERM_SSDEEP = 'SSDEEP'


@register_vocab
class ActionArgumentName(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:ActionArgumentNameVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ACCESS_MODE = 'Access Mode'
    TERM_APC_ADDRESS = 'APC Address'
    TERM_APC_MODE = 'APC Mode'
    TERM_API = 'API'
    TERM_APPLICATION_NAME = 'Application Name'
    TERM_BASE_ADDRESS = 'Base Address'
    TERM_CALLBACK_ADDRESS = 'Callback Address'
    TERM_CODE_ADDRESS = 'Code Address'
    TERM_COMMAND = 'Command'
    TERM_CONTROL_CODE = 'Control Code'
    TERM_CONTROL_PARAMETER = 'Control Parameter'
    TERM_CREATION_FLAGS = 'Creation Flags'
    TERM_DATABASE_NAME = 'Database Name'
    TERM_DELAY_TIME_MS = 'Delay Time (ms)'
    TERM_DESTINATION_ADDRESS = 'Destination Address'
    TERM_ERROR_CONTROL = 'Error Control'
    TERM_FILE_INFORMATION_CLASS = 'File Information Class'
    TERM_FLAGS = 'Flags'
    TERM_FUNCTION_ADDRESS = 'Function Address'
    TERM_FUNCTION_NAME = 'Function Name'
    TERM_FUNCTION_ORDINAL = 'Function Ordinal'
    TERM_HOOK_TYPE = 'Hook Type'
    TERM_HOSTNAME = 'Hostname'
    TERM_HOST_NAME = 'Host Name'
    TERM_INITIAL_OWNER = 'Initial Owner'
    TERM_MAPPING_OFFSET = 'Mapping Offset'
    TERM_NUMBER_OF_BYTES_PER_SEND = 'Number of Bytes Per Send'
    TERM_OPTIONS = 'Options'
    TERM_PARAMETER_ADDRESS = 'Parameter Address'
    TERM_PASSWORD = 'Password'
    TERM_PRIVILEGE_NAME = 'Privilege Name'
    TERM_PROTECTION = 'Protection'
    TERM_PROXY_BYPASS = 'Proxy Bypass'
    TERM_PROXY_NAME = 'Proxy Name'
    TERM_REASON = 'Reason'
    TERM_REQUESTED_VERSION = 'Requested Version'
    TERM_REQUEST_SIZE = 'Request Size'
    TERM_SERVER = 'Server'
    TERM_SERVICE_NAME = 'Service Name'
    TERM_SERVICE_STATE = 'Service State'
    TERM_SERVICE_TYPE = 'Service Type'
    TERM_SHARE_MODE = 'Share Mode'
    TERM_SHUTDOWN_FLAG = 'Shutdown Flag'
    TERM_SIZE_BYTES = 'Size (bytes)'
    TERM_SLEEP_TIME_MS = 'Sleep Time (ms)'
    TERM_SOURCE_ADDRESS = 'Source Address'
    TERM_STARTING_ADDRESS = 'Starting Address'
    TERM_SYSTEM_METRIC_INDEX = 'System Metric Index'
    TERM_TARGET_PID = 'Target PID'
    TERM_TRANSFER_FLAGS = 'Transfer Flags'
    TERM_USERNAME = 'Username'


@register_vocab
class ActionRelationshipType(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:ActionRelationshipTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_DEPENDENT_ON = 'Dependent_On'
    TERM_EQUIVALENT_TO = 'Equivalent_To'
    TERM_FOLLOWED_BY = 'Followed_By'
    TERM_INITIATED = 'Initiated'
    TERM_INITIATED_BY = 'Initiated_By'
    TERM_PRECEDED_BY = 'Preceded_By'
    TERM_RELATED_TO = 'Related_To'


@register_vocab
class ObjectRelationship(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:ObjectRelationshipVocab-1.1'
    _VOCAB_VERSION = '1.1'

    TERM_ALLOCATED = 'Allocated'
    TERM_ALLOCATED_BY = 'Allocated_By'
    TERM_BOUND = 'Bound'
    TERM_BOUND_BY = 'Bound_By'
    TERM_CHARACTERIZED_BY = 'Characterized_By'
    TERM_CHARACTERIZES = 'Characterizes'
    TERM_CHILD_OF = 'Child_Of'
    TERM_CLOSED = 'Closed'
    TERM_CLOSED_BY = 'Closed_By'
    TERM_COMPRESSED = 'Compressed'
    TERM_COMPRESSED_BY = 'Compressed_By'
    TERM_COMPRESSED_FROM = 'Compressed_From'
    TERM_COMPRESSED_INTO = 'Compressed_Into'
    TERM_CONNECTED_FROM = 'Connected_From'
    TERM_CONNECTED_TO = 'Connected_To'
    TERM_CONTAINED_WITHIN = 'Contained_Within'
    TERM_CONTAINS = 'Contains'
    TERM_COPIED = 'Copied'
    TERM_COPIED_BY = 'Copied_By'
    TERM_COPIED_FROM = 'Copied_From'
    TERM_COPIED_TO = 'Copied_To'
    TERM_CREATED = 'Created'
    TERM_CREATED_BY = 'Created_By'
    TERM_DECODED = 'Decoded'
    TERM_DECODED_BY = 'Decoded_By'
    TERM_DECOMPRESSED = 'Decompressed'
    TERM_DECOMPRESSED_BY = 'Decompressed_By'
    TERM_DECRYPTED = 'Decrypted'
    TERM_DECRYPTED_BY = 'Decrypted_By'
    TERM_DELETED = 'Deleted'
    TERM_DELETED_BY = 'Deleted_By'
    TERM_DELETED_FROM = 'Deleted_From'
    TERM_DOWNLOADED = 'Downloaded'
    TERM_DOWNLOADED_BY = 'Downloaded_By'
    TERM_DOWNLOADED_FROM = 'Downloaded_From'
    TERM_DOWNLOADED_TO = 'Downloaded_To'
    TERM_DROPPED = 'Dropped'
    TERM_DROPPED_BY = 'Dropped_By'
    TERM_ENCODED = 'Encoded'
    TERM_ENCODED_BY = 'Encoded_By'
    TERM_ENCRYPTED = 'Encrypted'
    TERM_ENCRYPTED_BY = 'Encrypted_By'
    TERM_ENCRYPTED_FROM = 'Encrypted_From'
    TERM_ENCRYPTED_TO = 'Encrypted_To'
    TERM_EXTRACTED_FROM = 'Extracted_From'
    TERM_FQDN_OF = 'FQDN_Of'
    TERM_FREED = 'Freed'
    TERM_FREED_BY = 'Freed_By'
    TERM_HOOKED = 'Hooked'
    TERM_HOOKED_BY = 'Hooked_By'
    TERM_INITIALIZED_BY = 'Initialized_By'
    TERM_INITIALIZED_TO = 'Initialized_To'
    TERM_INJECTED = 'Injected'
    TERM_INJECTED_AS = 'Injected_As'
    TERM_INJECTED_BY = 'Injected_By'
    TERM_INJECTED_INTO = 'Injected_Into'
    TERM_INSTALLED = 'Installed'
    TERM_INSTALLED_BY = 'Installed_By'
    TERM_JOINED = 'Joined'
    TERM_JOINED_BY = 'Joined_By'
    TERM_KILLED = 'Killed'
    TERM_KILLED_BY = 'Killed_By'
    TERM_LISTENED_ON = 'Listened_On'
    TERM_LISTENED_ON_BY = 'Listened_On_By'
    TERM_LOADED_FROM = 'Loaded_From'
    TERM_LOADED_INTO = 'Loaded_Into'
    TERM_LOCKED = 'Locked'
    TERM_LOCKED_BY = 'Locked_By'
    TERM_MAPPED_BY = 'Mapped_By'
    TERM_MAPPED_INTO = 'Mapped_Into'
    TERM_MERGED = 'Merged'
    TERM_MERGED_BY = 'Merged_By'
    TERM_MODIFIED_PROPERTIES_OF = 'Modified_Properties_Of'
    TERM_MONITORED = 'Monitored'
    TERM_MONITORED_BY = 'Monitored_By'
    TERM_MOVED = 'Moved'
    TERM_MOVED_BY = 'Moved_By'
    TERM_MOVED_FROM = 'Moved_From'
    TERM_MOVED_TO = 'Moved_To'
    TERM_OPENED = 'Opened'
    TERM_OPENED_BY = 'Opened_By'
    TERM_PACKED = 'Packed'
    TERM_PACKED_BY = 'Packed_By'
    TERM_PACKED_FROM = 'Packed_From'
    TERM_PACKED_INTO = 'Packed_Into'
    TERM_PARENT_OF = 'Parent_Of'
    TERM_PAUSED = 'Paused'
    TERM_PAUSED_BY = 'Paused_By'
    TERM_PREVIOUSLY_CONTAINED = 'Previously_Contained'
    TERM_PROPERTIES_MODIFIED_BY = 'Properties_Modified_By'
    TERM_PROPERTIES_QUERIED = 'Properties_Queried'
    TERM_PROPERTIES_QUERIED_BY = 'Properties_Queried_By'
    TERM_READ_FROM = 'Read_From'
    TERM_READ_FROM_BY = 'Read_From_By'
    TERM_RECEIVED = 'Received'
    TERM_RECEIVED_BY = 'Received_By'
    TERM_RECEIVED_FROM = 'Received_From'
    TERM_RECEIVED_VIA_UPLOAD = 'Received_Via_Upload'
    TERM_REDIRECTS_TO = 'Redirects_To'
    TERM_RELATED_TO = 'Related_To'
    TERM_RENAMED = 'Renamed'
    TERM_RENAMED_BY = 'Renamed_By'
    TERM_RENAMED_FROM = 'Renamed_From'
    TERM_RENAMED_TO = 'Renamed_To'
    TERM_RESOLVED_TO = 'Resolved_To'
    TERM_RESUMED = 'Resumed'
    TERM_RESUMED_BY = 'Resumed_By'
    TERM_ROOT_DOMAIN_OF = 'Root_Domain_Of'
    TERM_SEARCHED_FOR = 'Searched_For'
    TERM_SEARCHED_FOR_BY = 'Searched_For_By'
    TERM_SENT = 'Sent'
    TERM_SENT_BY = 'Sent_By'
    TERM_SENT_TO = 'Sent_To'
    TERM_SENT_VIA_UPLOAD = 'Sent_Via_Upload'
    TERM_SET_FROM = 'Set_From'
    TERM_SET_TO = 'Set_To'
    TERM_SUB_DOMAIN_OF = 'Sub-domain_Of'
    TERM_SUPRA_DOMAIN_OF = 'Supra-domain_Of'
    TERM_SUSPENDED = 'Suspended'
    TERM_SUSPENDED_BY = 'Suspended_By'
    TERM_UNHOOKED = 'Unhooked'
    TERM_UNHOOKED_BY = 'Unhooked_By'
    TERM_UNLOCKED = 'Unlocked'
    TERM_UNLOCKED_BY = 'Unlocked_By'
    TERM_UNPACKED = 'Unpacked'
    TERM_UNPACKED_BY = 'Unpacked_By'
    TERM_UPLOADED = 'Uploaded'
    TERM_UPLOADED_BY = 'Uploaded_By'
    TERM_UPLOADED_FROM = 'Uploaded_From'
    TERM_UPLOADED_TO = 'Uploaded_To'
    TERM_USED = 'Used'
    TERM_USED_BY = 'Used_By'
    TERM_VALUES_ENUMERATED = 'Values_Enumerated'
    TERM_VALUES_ENUMERATED_BY = 'Values_Enumerated_By'
    TERM_WRITTEN_TO_BY = 'Written_To_By'
    TERM_WROTE_TO = 'Wrote_To'


@register_vocab
class CharacterEncoding(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:CharacterEncodingVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ASCII = 'ASCII'
    TERM_UTF_16 = 'UTF-16'
    TERM_UTF_32 = 'UTF-32'
    TERM_UTF_8 = 'UTF-8'
    TERM_WINDOWS_1250 = 'Windows-1250'
    TERM_WINDOWS_1251 = 'Windows-1251'
    TERM_WINDOWS_1252 = 'Windows-1252'
    TERM_WINDOWS_1253 = 'Windows-1253'
    TERM_WINDOWS_1254 = 'Windows-1254'
    TERM_WINDOWS_1255 = 'Windows-1255'
    TERM_WINDOWS_1256 = 'Windows-1256'
    TERM_WINDOWS_1257 = 'Windows-1257'
    TERM_WINDOWS_1258 = 'Windows-1258'


@register_vocab
class ObjectState(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:ObjectStateVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_ACTIVE = 'Active'
    TERM_CLOSED = 'Closed'
    TERM_DOES_NOT_EXIST = 'Does Not Exist'
    TERM_EXISTS = 'Exists'
    TERM_INACTIVE = 'Inactive'
    TERM_LOCKED = 'Locked'
    TERM_OPEN = 'Open'
    TERM_STARTED = 'Started'
    TERM_STOPPED = 'Stopped'
    TERM_UNLOCKED = 'Unlocked'


@register_vocab
class ToolType(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:ToolTypeVocab-1.1'
    _VOCAB_VERSION = '1.1'

    TERM_ASSET_SCANNER = 'Asset Scanner'
    TERM_AV = 'AV'
    TERM_CONFIGURATION_SCANNER = 'Configuration Scanner'
    TERM_DBMS_MONITOR = 'DBMS Monitor'
    TERM_DIGITAL_FORENSICS = 'Digital Forensics'
    TERM_DYNAMIC_MALWARE_ANALYSIS = 'Dynamic Malware Analysis'
    TERM_FIREWALL = 'Firewall'
    TERM_GATEWAY = 'Gateway'
    TERM_HIDS = 'HIDS'
    TERM_HIPS = 'HIPS'
    TERM_INTELLIGENCE_SERVICE_PLATFORM = 'Intelligence Service Platform'
    TERM_NETWORK_CONFIGURATION_MANAGEMENT_TOOL = 'Network Configuration Management Tool'
    TERM_NETWORK_FLOW_CAPTURE_AND_ANALYSIS = 'Network Flow Capture and Analysis'
    TERM_NIDS = 'NIDS'
    TERM_NIPS = 'NIPS'
    TERM_PACKET_CAPTURE_AND_ANALYSIS = 'Packet Capture and Analysis'
    TERM_PROXY = 'Proxy'
    TERM_ROUTER = 'Router'
    TERM_SEM = 'SEM'
    TERM_SIM = 'SIM'
    TERM_SNMP_MIBS = 'SNMP/MIBs'
    TERM_STATIC_MALWARE_ANALYSIS = 'Static Malware Analysis'
    TERM_SYSTEM_CONFIGURATION_MANAGEMENT_TOOL = 'System Configuration Management Tool'
    TERM_VULNERABILITY_SCANNER = 'Vulnerability Scanner'


@register_vocab
class ActionName(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:ActionNameVocab-1.1'
    _VOCAB_VERSION = '1.1'

    TERM_ACCEPT_SOCKET_CONNECTION = 'Accept Socket Connection'
    TERM_ADD_CONNECTION_TO_NETWORK_SHARE = 'Add Connection to Network Share'
    TERM_ADD_NETWORK_SHARE = 'Add Network Share'
    TERM_ADD_SCHEDULED_TASK = 'Add Scheduled Task'
    TERM_ADD_SYSTEM_CALL_HOOK = 'Add System Call Hook'
    TERM_ADD_USER = 'Add User'
    TERM_ADD_WINDOWS_HOOK = 'Add Windows Hook'
    TERM_ALLOCATE_VIRTUAL_MEMORY_IN_PROCESS = 'Allocate Virtual Memory in Process'
    TERM_BIND_ADDRESS_TO_SOCKET = 'Bind Address to Socket'
    TERM_CHANGE_SERVICE_CONFIGURATION = 'Change Service Configuration'
    TERM_CHECK_FOR_REMOTE_DEBUGGER = 'Check for Remote Debugger'
    TERM_CLOSE_PORT = 'Close Port'
    TERM_CLOSE_REGISTRY_KEY = 'Close Registry Key'
    TERM_CLOSE_SOCKET = 'Close Socket'
    TERM_CONFIGURE_SERVICE = 'Configure Service'
    TERM_CONNECT_TO_IP = 'Connect to IP'
    TERM_CONNECT_TO_NAMED_PIPE = 'Connect to Named Pipe'
    TERM_CONNECT_TO_NETWORK_SHARE = 'Connect to Network Share'
    TERM_CONNECT_TO_SOCKET = 'Connect to Socket'
    TERM_CONNECT_TO_URL = 'Connect to URL'
    TERM_CONTROL_DRIVER = 'Control Driver'
    TERM_CONTROL_SERVICE = 'Control Service'
    TERM_COPY_FILE = 'Copy File'
    TERM_CREATE_DIALOG_BOX = 'Create Dialog Box'
    TERM_CREATE_DIRECTORY = 'Create Directory'
    TERM_CREATE_EVENT = 'Create Event'
    TERM_CREATE_FILE = 'Create File'
    TERM_CREATE_FILE_ALTERNATE_DATA_STREAM = 'Create File Alternate Data Stream'
    TERM_CREATE_FILE_MAPPING = 'Create File Mapping'
    TERM_CREATE_FILE_SYMBOLIC_LINK = 'Create File Symbolic Link'
    TERM_CREATE_HIDDEN_FILE = 'Create Hidden File'
    TERM_CREATE_MAILSLOT = 'Create Mailslot'
    TERM_CREATE_MODULE = 'Create Module'
    TERM_CREATE_MUTEX = 'Create Mutex'
    TERM_CREATE_NAMED_PIPE = 'Create Named Pipe'
    TERM_CREATE_PROCESS = 'Create Process'
    TERM_CREATE_PROCESS_AS_USER = 'Create Process as User'
    TERM_CREATE_REGISTRY_KEY = 'Create Registry Key'
    TERM_CREATE_REGISTRY_KEY_VALUE = 'Create Registry Key Value'
    TERM_CREATE_REMOTE_THREAD_IN_PROCESS = 'Create Remote Thread in Process'
    TERM_CREATE_SERVICE = 'Create Service'
    TERM_CREATE_SOCKET = 'Create Socket'
    TERM_CREATE_SYMBOLIC_LINK = 'Create Symbolic Link'
    TERM_CREATE_THREAD = 'Create Thread'
    TERM_CREATE_WINDOW = 'Create Window'
    TERM_DELETE_DIRECTORY = 'Delete Directory'
    TERM_DELETE_FILE = 'Delete File'
    TERM_DELETE_NAMED_PIPE = 'Delete Named Pipe'
    TERM_DELETE_NETWORK_SHARE = 'Delete Network Share'
    TERM_DELETE_REGISTRY_KEY = 'Delete Registry Key'
    TERM_DELETE_REGISTRY_KEY_VALUE = 'Delete Registry Key Value'
    TERM_DELETE_SERVICE = 'Delete Service'
    TERM_DELETE_USER = 'Delete User'
    TERM_DISCONNECT_FROM_NAMED_PIPE = 'Disconnect from Named Pipe'
    TERM_DISCONNECT_FROM_NETWORK_SHARE = 'Disconnect from Network Share'
    TERM_DISCONNECT_FROM_SOCKET = 'Disconnect from Socket'
    TERM_DOWNLOAD_FILE = 'Download File'
    TERM_ENUMERATE_DLLS = 'Enumerate DLLs'
    TERM_ENUMERATE_NETWORK_SHARES = 'Enumerate Network Shares'
    TERM_ENUMERATE_PROCESSES = 'Enumerate Processes'
    TERM_ENUMERATE_PROTOCOLS = 'Enumerate Protocols'
    TERM_ENUMERATE_REGISTRY_KEY_SUBKEYS = 'Enumerate Registry Key Subkeys'
    TERM_ENUMERATE_REGISTRY_KEY_VALUES = 'Enumerate Registry Key Values'
    TERM_ENUMERATE_SERVICES = 'Enumerate Services'
    TERM_ENUMERATE_SYSTEM_HANDLES = 'Enumerate System Handles'
    TERM_ENUMERATE_THREADS = 'Enumerate Threads'
    TERM_ENUMERATE_THREADS_IN_PROCESS = 'Enumerate Threads in Process'
    TERM_ENUMERATE_USERS = 'Enumerate Users'
    TERM_ENUMERATE_WINDOWS = 'Enumerate Windows'
    TERM_FIND_FILE = 'Find File'
    TERM_FIND_WINDOW = 'Find Window'
    TERM_FLUSH_PROCESS_INSTRUCTION_CACHE = 'Flush Process Instruction Cache'
    TERM_FREE_LIBRARY = 'Free Library'
    TERM_FREE_PROCESS_VIRTUAL_MEMORY = 'Free Process Virtual Memory'
    TERM_GET_DISK_FREE_SPACE = 'Get Disk Free Space'
    TERM_GET_DISK_TYPE = 'Get Disk Type'
    TERM_GET_ELAPSED_SYSTEM_UP_TIME = 'Get Elapsed System Up Time'
    TERM_GET_FILE_ATTRIBUTES = 'Get File Attributes'
    TERM_GET_FUNCTION_ADDRESS = 'Get Function Address'
    TERM_GET_HOST_BY_ADDRESS = 'Get Host By Address'
    TERM_GET_HOST_BY_NAME = 'Get Host By Name'
    TERM_GET_HOST_NAME = 'Get Host Name'
    TERM_GET_LIBRARY_FILE_NAME = 'Get Library File Name'
    TERM_GET_LIBRARY_HANDLE = 'Get Library Handle'
    TERM_GET_NETBIOS_NAME = 'Get NetBIOS Name'
    TERM_GET_PROCESSES_SNAPSHOT = 'Get Processes Snapshot'
    TERM_GET_PROCESS_CURRENT_DIRECTORY = 'Get Process Current Directory'
    TERM_GET_PROCESS_ENVIRONMENT_VARIABLE = 'Get Process Environment Variable'
    TERM_GET_PROCESS_STARTUP_INFORMATION = 'Get Process Startup Information'
    TERM_GET_REGISTRY_KEY_ATTRIBUTES = 'Get Registry Key Attributes'
    TERM_GET_SERVICE_STATUS = 'Get Service Status'
    TERM_GET_SYSTEM_GLOBAL_FLAGS = 'Get System Global Flags'
    TERM_GET_SYSTEM_HOST_NAME = 'Get System Host Name'
    TERM_GET_SYSTEM_LOCAL_TIME = 'Get System Local Time'
    TERM_GET_SYSTEM_NETBIOS_NAME = 'Get System NetBIOS Name'
    TERM_GET_SYSTEM_NETWORK_PARAMETERS = 'Get System Network Parameters'
    TERM_GET_SYSTEM_TIME = 'Get System Time'
    TERM_GET_THREAD_CONTEXT = 'Get Thread Context'
    TERM_GET_THREAD_USERNAME = 'Get Thread Username'
    TERM_GET_USERNAME = 'Get Username'
    TERM_GET_USER_ATTRIBUTES = 'Get User Attributes'
    TERM_GET_WINDOWS_DIRECTORY = 'Get Windows Directory'
    TERM_GET_WINDOWS_SYSTEM_DIRECTORY = 'Get Windows System Directory'
    TERM_GET_WINDOWS_TEMPORARY_FILES_DIRECTORY = 'Get Windows Temporary Files Directory'
    TERM_HIDE_WINDOW = 'Hide Window'
    TERM_IMPERSONATE_PROCESS = 'Impersonate Process'
    TERM_IMPERSONATE_THREAD = 'Impersonate Thread'
    TERM_INJECT_MEMORY_PAGE = 'Inject Memory Page'
    TERM_KILL_PROCESS = 'Kill Process'
    TERM_KILL_THREAD = 'Kill Thread'
    TERM_KILL_WINDOW = 'Kill Window'
    TERM_LISTEN_ON_PORT = 'Listen on Port'
    TERM_LISTEN_ON_SOCKET = 'Listen on Socket'
    TERM_LOAD_AND_CALL_DRIVER = 'Load and Call Driver'
    TERM_LOAD_DRIVER = 'Load Driver'
    TERM_LOAD_LIBRARY = 'Load Library'
    TERM_LOAD_MODULE = 'Load Module'
    TERM_LOCK_FILE = 'Lock File'
    TERM_LOGON_AS_USER = 'Logon as User'
    TERM_MAP_FILE = 'Map File'
    TERM_MAP_LIBRARY = 'Map Library'
    TERM_MAP_VIEW_OF_FILE = 'Map View of File'
    TERM_MODIFY_FILE = 'Modify File'
    TERM_MODIFY_NAMED_PIPE = 'Modify Named Pipe'
    TERM_MODIFY_PROCESS = 'Modify Process'
    TERM_MODIFY_REGISTRY_KEY = 'Modify Registry Key'
    TERM_MODIFY_REGISTRY_KEY_VALUE = 'Modify Registry Key Value'
    TERM_MODIFY_SERVICE = 'Modify Service'
    TERM_MONITOR_REGISTRY_KEY = 'Monitor Registry Key'
    TERM_MOVE_FILE = 'Move File'
    TERM_OPEN_FILE = 'Open File'
    TERM_OPEN_FILE_MAPPING = 'Open File Mapping'
    TERM_OPEN_MUTEX = 'Open Mutex'
    TERM_OPEN_PORT = 'Open Port'
    TERM_OPEN_PROCESS = 'Open Process'
    TERM_OPEN_REGISTRY_KEY = 'Open Registry Key'
    TERM_OPEN_SERVICE = 'Open Service'
    TERM_OPEN_SERVICE_CONTROL_MANAGER = 'Open Service Control Manager'
    TERM_PROTECT_VIRTUAL_MEMORY = 'Protect Virtual Memory'
    TERM_QUERY_DISK_ATTRIBUTES = 'Query Disk Attributes'
    TERM_QUERY_DNS = 'Query DNS'
    TERM_QUERY_PROCESS_VIRTUAL_MEMORY = 'Query Process Virtual Memory'
    TERM_QUEUE_APC_IN_THREAD = 'Queue APC in Thread'
    TERM_READ_FILE = 'Read File'
    TERM_READ_FROM_NAMED_PIPE = 'Read From Named Pipe'
    TERM_READ_FROM_PROCESS_MEMORY = 'Read From Process Memory'
    TERM_READ_REGISTRY_KEY_VALUE = 'Read Registry Key Value'
    TERM_RECEIVE_DATA_ON_SOCKET = 'Receive Data on Socket'
    TERM_RECEIVE_EMAIL_MESSAGE = 'Receive Email Message'
    TERM_RELEASE_MUTEX = 'Release Mutex'
    TERM_RENAME_FILE = 'Rename File'
    TERM_REVERT_THREAD_TO_SELF = 'Revert Thread to Self'
    TERM_SEND_CONTROL_CODE_TO_FILE = 'Send Control Code to File'
    TERM_SEND_CONTROL_CODE_TO_PIPE = 'Send Control Code to Pipe'
    TERM_SEND_CONTROL_CODE_TO_SERVICE = 'Send Control Code to Service'
    TERM_SEND_DATA_ON_SOCKET = 'Send Data on Socket'
    TERM_SEND_DATA_TO_ADDRESS_ON_SOCKET = 'Send Data to Address on Socket'
    TERM_SEND_DNS_QUERY = 'Send DNS Query'
    TERM_SEND_EMAIL_MESSAGE = 'Send Email Message'
    TERM_SEND_ICMP_REQUEST = 'Send ICMP Request'
    TERM_SEND_REVERSE_DNS_QUERY = 'Send Reverse DNS Query'
    TERM_SET_FILE_ATTRIBUTES = 'Set File Attributes'
    TERM_SET_NETBIOS_NAME = 'Set NetBIOS Name'
    TERM_SET_PROCESS_CURRENT_DIRECTORY = 'Set Process Current Directory'
    TERM_SET_PROCESS_ENVIRONMENT_VARIABLE = 'Set Process Environment Variable'
    TERM_SET_SYSTEM_GLOBAL_FLAGS = 'Set System Global Flags'
    TERM_SET_SYSTEM_HOST_NAME = 'Set System Host Name'
    TERM_SET_SYSTEM_TIME = 'Set System Time'
    TERM_SET_THREAD_CONTEXT = 'Set Thread Context'
    TERM_SHOW_WINDOW = 'Show Window'
    TERM_SHUTDOWN_SYSTEM = 'Shutdown System'
    TERM_SLEEP_PROCESS = 'Sleep Process'
    TERM_SLEEP_SYSTEM = 'Sleep System'
    TERM_START_SERVICE = 'Start Service'
    TERM_UNLOAD_DRIVER = 'Unload Driver'
    TERM_UNLOAD_MODULE = 'Unload Module'
    TERM_UNLOCK_FILE = 'Unlock File'
    TERM_UNMAP_FILE = 'Unmap File'
    TERM_UPLOAD_FILE = 'Upload File'
    TERM_WRITE_TO_FILE = 'Write to File'
    TERM_WRITE_TO_PROCESS_VIRTUAL_MEMORY = 'Write to Process Virtual Memory'


@register_vocab
class InformationSourceType(VocabString):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/vocabularies-2'
    _XSI_TYPE = 'cyboxVocabs:InformationSourceTypeVocab-1.0'
    _VOCAB_VERSION = '1.0'

    TERM_APPLICATION_FRAMEWORK = 'Application Framework'
    TERM_APPLICATION_LOGS = 'Application Logs'
    TERM_COMM_LOGS = 'Comm Logs'
    TERM_DBMS_LOG = 'DBMS Log'
    TERM_FRAMEWORKS = 'Frameworks'
    TERM_HELP_DESK = 'Help Desk'
    TERM_IAVM = 'IAVM'
    TERM_INCIDENT_MANAGEMENT = 'Incident Management'
    TERM_OS_DEVICE_DRIVER_APIS = 'OS/Device Driver APIs'
    TERM_TPM = 'TPM'
    TERM_VM_HYPERVISOR = 'VM Hypervisor'
    TERM_WEB_LOGS = 'Web Logs'



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.process_object as process_binding
from cybox.common import ObjectProperties, String, DateTime, UnsignedInteger, Duration, EnvironmentVariableList, ExtractedFeatures
from cybox.objects.network_connection_object import NetworkConnection
from cybox.objects.port_object import Port


class PortList(entities.EntityList):
    _binding = process_binding
    _binding_class = process_binding.PortListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/process-2"
    port = fields.TypedField("Port", Port, multiple=True)


class NetworkConnectionList(entities.EntityList):
    _binding = process_binding
    _binding_class = process_binding.NetworkConnectionListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/process-2"
    network_connection = fields.TypedField("Network_Connection", NetworkConnection, multiple=True)


class ChildPIDList(entities.EntityList):
    _binding = process_binding
    _binding_class = process_binding.ChildPIDListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/process-2"
    child_pid = fields.TypedField("Child_PID", UnsignedInteger, multiple=True)


class ArgumentList(entities.EntityList):
    _binding = process_binding
    _binding_class = process_binding.ArgumentListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/process-2"
    argument = fields.TypedField("Argument", String, multiple=True)


class ImageInfo(entities.Entity):
    _binding = process_binding
    _binding_class = process_binding.ImageInfoType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/process-2"

    file_name = fields.TypedField("File_Name", String)
    command_line = fields.TypedField("Command_Line", String)
    current_directory = fields.TypedField("Current_Directory", String)
    path = fields.TypedField("Path", String)


class Process(ObjectProperties):
    _binding = process_binding
    _binding_class = process_binding.ProcessObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/process-2"
    _XSI_NS = "ProcessObj"
    _XSI_TYPE = "ProcessObjectType"

    pid = fields.TypedField("PID", UnsignedInteger)
    name = fields.TypedField("Name", String)
    creation_time = fields.TypedField("Creation_Time", DateTime)
    parent_pid = fields.TypedField("Parent_PID", UnsignedInteger)
    child_pid_list = fields.TypedField("Child_PID_List", ChildPIDList)
    image_info = fields.TypedField("Image_Info", ImageInfo)
    argument_list = fields.TypedField("Argument_List", ArgumentList)
    environment_variable_list = fields.TypedField("Environment_Variable_List", EnvironmentVariableList)
    kernel_time = fields.TypedField("Kernel_Time", Duration)
    port_list = fields.TypedField("Port_List", PortList)
    network_connection_list = fields.TypedField("Network_Connection_List", NetworkConnectionList)
    start_time = fields.TypedField("Start_Time", DateTime)
    #status TODO: Add support
    username = fields.TypedField("Username", String)
    user_time = fields.TypedField("User_Time", Duration)
    extracted_features = fields.TypedField("Extracted_Features", ExtractedFeatures)
    is_hidden = fields.TypedField("is_hidden")





# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_network_share_object as win_network_share_binding
from cybox.common import ObjectProperties, String, NonNegativeInteger


class WinNetworkShare(ObjectProperties):
    _binding = win_network_share_binding
    _binding_class = win_network_share_binding.WindowsNetworkShareObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-network-share-2"
    _XSI_NS = "WinNetworkShareObj"
    _XSI_TYPE = "WindowsNetworkShareObjectType"

    access_read = fields.TypedField("ACCESS_READ")
    access_write = fields.TypedField("ACCESS_WRITE")
    access_create = fields.TypedField("ACCESS_CREATE")
    access_exec = fields.TypedField("ACCESS_EXEC")
    access_delete = fields.TypedField("ACCESS_DELETE")
    access_atrib = fields.TypedField("ACCESS_ATRIB")
    access_perm = fields.TypedField("ACCESS_PERM")
    access_all = fields.TypedField("ACCESS_ALL")
    current_uses = fields.TypedField("Current_Uses", NonNegativeInteger)
    local_path = fields.TypedField("Local_Path", String)
    max_uses = fields.TypedField("Max_Uses", NonNegativeInteger)
    netname = fields.TypedField("Netname", String)
    type_ = fields.TypedField("Type", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.pipe_object import Pipe
from cybox.test.objects import ObjectTestCase


class TestPipe(ObjectTestCase, unittest.TestCase):
    object_type = "PipeObjectType"
    klass = Pipe

    _full_dict = {
        'name': u("pipe1"),
        'named': True,
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_pipe_object as win_pipe_binding
from cybox.objects.pipe_object import Pipe
from cybox.objects.win_handle_object import WinHandle
from cybox.common import String, NonNegativeInteger, HexBinary


class WinPipe(Pipe):
    _binding = win_pipe_binding
    _binding_class = win_pipe_binding.WindowsPipeObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-pipe-2'
    _XSI_NS = "WinPipeObj"
    _XSI_TYPE = "WindowsPipeObjectType"

    default_time_out = fields.TypedField("Default_Time_Out", NonNegativeInteger)
    handle = fields.TypedField("Handle", WinHandle)
    in_buffer_size = fields.TypedField("In_Buffer_Size", NonNegativeInteger)
    max_instances = fields.TypedField("Max_Instances", NonNegativeInteger)
    open_mode = fields.TypedField("Open_Mode", HexBinary)
    out_buffer_size = fields.TypedField("Out_Buffer_Size", NonNegativeInteger)
    pipe_mode = fields.TypedField("Pipe_Mode", HexBinary)
    security_attributes = fields.TypedField("Security_Attributes", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.gui_object as gui_binding
from cybox.common import ObjectProperties, Integer


class GUI(ObjectProperties):
    _binding = gui_binding
    _binding_class = gui_binding.GUIObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/gui-2"
    _XSI_NS = "GUIObj"
    _XSI_TYPE = "GUIObjectType"

    height = fields.TypedField("Height", Integer)
    width = fields.TypedField("Width", Integer)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import binascii
import unittest

from mixbox.vendor import six
from mixbox.vendor.six import u

from cybox.common import ExtractedString, Hash
from cybox.test import EntityTestCase

STRING = u("This is a string")
HEX_STRING = six.text_type(binascii.hexlify(STRING.encode("ascii")))

class TestExtractedString(EntityTestCase, unittest.TestCase):
    klass = ExtractedString

    _full_dict = {
        'encoding': u("UTF-8"),
        'string_value': STRING,
        'byte_string_value': HEX_STRING,
        'hashes': [{'type': Hash.TYPE_MD5}],
        'address': u("1a2b"),
        'length': len(STRING),
        'language': u("English"),
        'english_translation': STRING,
    }


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""Creates the CybOX content for CybOX_Simple_Email_Pattern.xml
"""

from cybox.core import Observables
from cybox.objects.email_message_object import EmailMessage


def main():
    m = EmailMessage()
    m.from_ = ["attacker@example.com",
               "attacker1@example.com",
               "attacker@bad.example.com"]
    m.from_.condition = "Equals"
    m.subject = "New modifications to the specification"
    m.subject.condition = "Equals"

    print(Observables(m).to_xml(encoding=None))

if __name__ == "__main__":
    main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.win_hook_object import WinHook

from cybox.test import EntityTestCase, round_trip
from cybox.test.objects import ObjectTestCase


class TestWinHook(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsHookObjectType"
    klass = WinHook

    _full_dict = {
        'type': u("Test Hook"),
        #TODO: add 'handle'
        'hooking_function_name': u("test_function"),
        #TODO: add 'hooking_module'
        'thread_id': 2,
        'xsi:type': object_type,
    }

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_kernel_hook_object as win_kernel_hook_binding
from cybox.common import (DigitalSignature, ObjectProperties, String,
        UnsignedLong)


class WinKernelHook(ObjectProperties):
    _binding = win_kernel_hook_binding
    _binding_class = win_kernel_hook_binding.WindowsKernelHookObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-kernel-hook-2"
    _XSI_NS = "WinKernelHookObj"
    _XSI_TYPE = "WindowsKernelHookObjectType"

    digital_signature_hooking = fields.TypedField("Digital_Signature_Hooking",
                                                 DigitalSignature)
    digital_signature_hooked = fields.TypedField("Digital_Signature_Hooked",
                                                DigitalSignature)
    hooking_address = fields.TypedField("Hooking_Address", UnsignedLong)
    hook_description = fields.TypedField("Hook_Description", String)
    hooked_function = fields.TypedField("Hooked_Function", String)
    hooked_module = fields.TypedField("Hooked_Module", String)
    hooking_module = fields.TypedField("Hooking_Module", String)
    type_ = fields.TypedField("Type", String)



#!/usr/bin/env python

# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""Creates the CybOX content for CybOX_Simple_File_Instance.xml
"""

from cybox.common import Hash
from cybox.core import Observable, Observables
from cybox.objects.file_object import File


def main():
    h = Hash("a7a0390e99406f8975a1895860f55f2f")

    f = File()
    f.file_name = "bad_file24.exe"
    f.file_path = "AppData\Mozilla"
    f.file_extension = ".exe"
    f.size_in_bytes = 3282
    f.add_hash(h)

    o = Observable(f)
    o.description = "This observable specifies a specific file observation."

    print(Observables(o).to_xml(encoding=None))

if __name__ == "__main__":
    main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.win_waitable_timer_object import WinWaitableTimer

from cybox.test import EntityTestCase, round_trip
from cybox.test.objects import ObjectTestCase


class TestWinWaitableTimer(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsWaitableTimerObjectType"
    klass = WinWaitableTimer

    _full_dict = {
        'security_attributes': u("timer attributes"),
        'name': u("timer name"),
        'type': u("timer type"),
        'xsi:type': object_type,
    }

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.user_account_object import UserAccount
from cybox.test.objects import ObjectTestCase


class TestUserAccount(ObjectTestCase, unittest.TestCase):
    object_type = "UserAccountObjectType"
    klass = UserAccount

    _full_dict = {
        # Account-specific fields
        'disabled': False,
        'locked_out': True,
        'description': u('An account'),
        'domain': u('ADMIN'),
        # UserAccount-specific fields
        # (cannot test group_list of privilege_list since
        # they are abstract)
        'password_required': True,
        'full_name': u("Guido van Rossum"),
        'home_directory': u("/home/guido/"),
        'last_login': "2001-01-01T06:56:50+04:00",
        'script_path': u("/bin/bash"),
        'username': u("guido"),
        'user_password_age': u("P90D"),
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.core import Event
from cybox.test import EntityTestCase


class TestEvent(EntityTestCase, unittest.TestCase):
    klass = Event

    _full_dict = {
        'id': "example:Event-1",
        'idref': "example:Event-2",
        'type': u("Port Scan"),
        'description': u("This is a test event"),
        'observation_method': {'class': "Network"},
        'frequency': {'rate': 1.0},
        'actions': [
            {'idref': "example:Action-5"},
            {'idref': "example:Action-6"},
        ],
        # Once the choice is implemented, this won't work
        'event': [
            {'idref': "example:Event-A"},
            {'idref': "example:Event-B"},
        ]
    }

    def test_empty_recursive_event(self):
        e = Event()
        e.description = "Foo"
        e.event = None
        self.assertTrue(b"Foo" in e.to_xml())


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.cybox_common as common_binding
from cybox.common import String


class EnvironmentVariable(entities.Entity):
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'
    _binding = common_binding
    _binding_class = _binding.EnvironmentVariableType

    name = fields.TypedField("Name", String)
    value = fields.TypedField("Value", String)


class EnvironmentVariableList(entities.EntityList):
    _binding_class = common_binding.EnvironmentVariableListType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'
    environment_variable = fields.TypedField("Environment_Variable", EnvironmentVariable, multiple=True)



from sphinx.ext.autodoc import AttributeDocumenter, ClassDocumenter
from sphinx.util.pycompat import class_types

from mixbox import entities
from mixbox import fields


def make_class_ref(cls):
    return ":py:class:`%s.%s`" % (cls.__module__, cls.__name__)


class EntityDocumenter(ClassDocumenter):

    objtype = "class"
    priority = 99

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return isinstance(member, class_types) and \
               issubclass(member, entities.Entity)

    def add_content(self, more_content, no_docstring=False):
        ClassDocumenter.add_content(self, more_content, no_docstring)
        obj = self.object

        try:
            binding_class = make_class_ref(obj._binding_class)
        except AttributeError:
            binding_class = "<undefined>"
        self.add_line("|  XML binding class: %s\n" % binding_class, "<autoentity>")


class TypedFieldDocumenter(AttributeDocumenter):

    objtype = "attribute"
    priority = 99

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return isinstance(member, fields.TypedField)

    def add_content(self, more_content, no_docstring=False):
        AttributeDocumenter.add_content(self, more_content, no_docstring)
        if self.object.multiple:
            self.add_line("|  (List of values permitted)", "<autoentity>")
        type_ = self.object.type_
        if type_:
            self.add_line("|  Type: %s\n" % make_class_ref(type_), "<autoentity>")
        self.add_line("|  XML Binding class name: ``%s``\n" % self.object.name, "<autoentity>")
        self.add_line("|  Dictionary key name: ``%s``\n" % self.object.key_name, "<autoentity>")


def setup(app):
    app.add_autodocumenter(EntityDocumenter)
    app.add_autodocumenter(TypedFieldDocumenter)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.win_task_object as win_task_binding
from cybox.common import (Base64Binary, DateTime, Duration, HashList, Long,
        ObjectProperties, String, UnsignedLong)
from cybox.objects.email_message_object import EmailMessage


class Trigger(entities.Entity):
    _binding = win_task_binding
    _binding_class = win_task_binding.TriggerType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-task-2'

    trigger_begin = fields.TypedField("Trigger_Begin", DateTime)
    trigger_delay = fields.TypedField("Trigger_Delay", Duration)
    trigger_end = fields.TypedField("Trigger_End", DateTime)
    trigger_frequency = fields.TypedField("Trigger_Frequency", String)
    trigger_max_run_time = fields.TypedField("Trigger_Max_Run_Time", Duration)
    trigger_session_change_type = fields.TypedField(
                                    "Trigger_Session_Change_Type", String)
    trigger_type = fields.TypedField("Trigger_Type", String)


class TriggerList(entities.EntityList):
    _binding = win_task_binding
    _binding_class = win_task_binding.TriggerListType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-task-2'
    trigger = fields.TypedField("Trigger", Trigger, multiple=True)


class IComHandlerAction(entities.Entity):
    _binding = win_task_binding
    _binding_class = win_task_binding.IComHandlerActionType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-task-2'

    com_data = fields.TypedField("COM_Data", String)
    com_class_id = fields.TypedField("COM_Class_ID", String)


class IExecAction(entities.Entity):
    _binding = win_task_binding
    _binding_class = win_task_binding.IExecActionType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-task-2'

    exec_arguments = fields.TypedField("Exec_Arguments", String)
    exec_program_path = fields.TypedField("Exec_Program_Path", String)
    exec_working_directory = fields.TypedField("Exec_Working_Directory", String)
    exec_program_hashes = fields.TypedField("Exec_Program_Hashes", HashList)


class IShowMessageAction(entities.Entity):
    _binding = win_task_binding
    _binding_class = win_task_binding.IShowMessageActionType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-task-2'

    show_message_body = fields.TypedField("Show_Message_Body", String)
    show_message_title = fields.TypedField("Show_Message_Title", String)


class TaskAction(entities.Entity):
    _binding = win_task_binding
    _binding_class = win_task_binding.TaskActionType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-task-2'

    action_type = fields.TypedField("Action_Type", String)
    action_id = fields.TypedField("Action_ID", String)
    iemailaction = fields.TypedField("IEmailAction", EmailMessage)
    icomhandleraction = fields.TypedField("IComHandlerAction",
                                         IComHandlerAction)
    iexecaction = fields.TypedField("IExecAction", IExecAction)
    ishowmessageaction = fields.TypedField("IShowMessageAction",
                                          IShowMessageAction)


class TaskActionList(entities.EntityList):
    _binding = win_task_binding
    _binding_class = win_task_binding.TaskActionListType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-task-2'
    action = fields.TypedField("Action", TaskAction, multiple=True)


class WinTask(ObjectProperties):
    _binding = win_task_binding
    _binding_class = win_task_binding.WindowsTaskObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-task-2'
    _XSI_NS = "WinTaskObj"
    _XSI_TYPE = "WindowsTaskObjectType"

    status = fields.TypedField("Status", String)
    priority = fields.TypedField("Priority", String)
    name = fields.TypedField("Name", String)
    application_name = fields.TypedField("Application_Name", String)
    parameters = fields.TypedField("Parameters", String)
    flags = fields.TypedField("Flags", String)
    account_name = fields.TypedField("Account_Name", String)
    account_run_level = fields.TypedField("Account_Run_Level", String)
    account_logon_type = fields.TypedField("Account_Logon_Type", String)
    creator = fields.TypedField("Creator", String)
    creation_date = fields.TypedField("Creation_Date", DateTime)
    most_recent_run_time = fields.TypedField("Most_Recent_Run_Time", DateTime)
    exit_code = fields.TypedField("Exit_Code", Long)
    max_run_time = fields.TypedField("Max_Run_Time", UnsignedLong)
    next_run_time = fields.TypedField("Next_Run_Time", DateTime)
    action_list = fields.TypedField("Action_List", TaskActionList)
    trigger_list = fields.TypedField("Trigger_List", TriggerList)
    comment = fields.TypedField("Comment", String)
    working_directory = fields.TypedField("Working_Directory", String)
    work_item_data = fields.TypedField("Work_Item_Data", Base64Binary)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_memory_page_region_object as win_memory_page_region_binding
from cybox.common import String, HashList, UnsignedLong, HexBinary, ExtractedFeatures
from cybox.objects.memory_object import Memory


class WinMemoryPageRegion(Memory):
    _binding = win_memory_page_region_binding
    _binding_class = win_memory_page_region_binding.WindowsMemoryPageRegionObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-memory-page-region-2"
    _XSI_NS = "WinMemoryPageRegionObj"
    _XSI_TYPE = "WindowsMemoryPageRegionObjectType"

    is_injected = fields.TypedField("is_injected")
    is_mapped = fields.TypedField("is_mapped")
    is_protected = fields.TypedField("is_protected")
    is_volatile = fields.TypedField("is_volatile")

    hashes = fields.TypedField("Hashes", HashList)
    name = fields.TypedField("Name", String)
    memory_source = fields.TypedField("Memory_Source", String)
    region_size = fields.TypedField("Region_Size", UnsignedLong)
    block_type = fields.TypedField("Block_Type", String)
    region_start_address = fields.TypedField("Region_Start_Address", HexBinary)
    region_end_address = fields.TypedField("Region_End_Address", HexBinary)
    extracted_features = fields.TypedField("Extracted_Features", ExtractedFeatures)
    allocation_base_address = fields.TypedField("Allocation_Base_Address", HexBinary)

    type_ = fields.TypedField("Type", String)
    allocation_protect = fields.TypedField("Allocation_Protect", String)
    state = fields.TypedField("State", String)
    protect = fields.TypedField("Protect", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import account_object
from . import port_object


class FullyQualifiedNameType(GeneratedsSuper):
    """The FullyQualifiedNameType type refers to the fully qualified
    name(s) of the Windows computer account."""

    subclass = None
    superclass = None
    def __init__(self, NetBEUI_Name=None, Full_Name=None):
        self.NetBEUI_Name = NetBEUI_Name
        self.Full_Name = Full_Name
    def factory(*args_, **kwargs_):
        if FullyQualifiedNameType.subclass:
            return FullyQualifiedNameType.subclass(*args_, **kwargs_)
        else:
            return FullyQualifiedNameType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_NetBEUI_Name(self): return self.NetBEUI_Name
    def set_NetBEUI_Name(self, NetBEUI_Name): self.NetBEUI_Name = NetBEUI_Name
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Full_Name(self): return self.Full_Name
    def set_Full_Name(self, Full_Name): self.Full_Name = Full_Name
    def hasContent_(self):
        if (
            self.NetBEUI_Name is not None or
            self.Full_Name is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='FullyQualifiedNameType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='FullyQualifiedNameType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinComputerAccountObj:', name_='FullyQualifiedNameType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='FullyQualifiedNameType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.NetBEUI_Name is not None:
            self.NetBEUI_Name.export(lwrite, level, 'WinComputerAccountObj:', name_='NetBEUI_Name', pretty_print=pretty_print)
        if self.Full_Name is not None:
            self.Full_Name.export(lwrite, level, 'WinComputerAccountObj:', name_='Full_Name', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'NetBEUI_Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_NetBEUI_Name(obj_)
        elif nodeName_ == 'Full_Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Full_Name(obj_)
# end class FullyQualifiedNameType

class KerberosType(GeneratedsSuper):
    """The KerberosType type specifies the Kerberos authentication protocol
    specific Object properties for the Windows computer account."""

    subclass = None
    superclass = None
    def __init__(self, Delegation=None, Ticket=None):
        self.Delegation = Delegation
        self.Ticket = Ticket
    def factory(*args_, **kwargs_):
        if KerberosType.subclass:
            return KerberosType.subclass(*args_, **kwargs_)
        else:
            return KerberosType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Delegation(self): return self.Delegation
    def set_Delegation(self, Delegation): self.Delegation = Delegation
    def get_Ticket(self): return self.Ticket
    def set_Ticket(self, Ticket): self.Ticket = Ticket
    def validate_UnsignedLongObjectPropertyType(self, value):
        # Validate type cybox_common.UnsignedLongObjectPropertyType, a restriction on None.
        pass
    def hasContent_(self):
        if (
            self.Delegation is not None or
            self.Ticket is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='KerberosType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='KerberosType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinComputerAccountObj:', name_='KerberosType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='KerberosType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Delegation is not None:
            self.Delegation.export(lwrite, level, 'WinComputerAccountObj:', name_='Delegation', pretty_print=pretty_print)
        if self.Ticket is not None:
            self.Ticket.export(lwrite, level, 'WinComputerAccountObj:', name_='Ticket', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Delegation':
            obj_ = KerberosDelegationType.factory()
            obj_.build(child_)
            self.set_Delegation(obj_)
        elif nodeName_ == 'Ticket':
            obj_ = cybox_common.UnsignedLongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Ticket(obj_)
# end class KerberosType

class KerberosDelegationType(GeneratedsSuper):
    """The Delegation field specifies the Kerberos delegation used for the
    Windows computer account."""

    subclass = None
    superclass = None
    def __init__(self, Bitmask=None, Service=None):
        self.Bitmask = Bitmask
        self.Service = Service
    def factory(*args_, **kwargs_):
        if KerberosDelegationType.subclass:
            return KerberosDelegationType.subclass(*args_, **kwargs_)
        else:
            return KerberosDelegationType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Bitmask(self): return self.Bitmask
    def set_Bitmask(self, Bitmask): self.Bitmask = Bitmask
    def validate_HexBinaryObjectPropertyType(self, value):
        # Validate type cybox_common.HexBinaryObjectPropertyType, a restriction on None.
        pass
    def get_Service(self): return self.Service
    def set_Service(self, Service): self.Service = Service
    def hasContent_(self):
        if (
            self.Bitmask is not None or
            self.Service is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='KerberosDelegationType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='KerberosDelegationType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinComputerAccountObj:', name_='KerberosDelegationType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='KerberosDelegationType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Bitmask is not None:
            self.Bitmask.export(lwrite, level, 'WinComputerAccountObj:', name_='Bitmask', pretty_print=pretty_print)
        if self.Service is not None:
            self.Service.export(lwrite, level, 'WinComputerAccountObj:', name_='Service', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Bitmask':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Bitmask(obj_)
        elif nodeName_ == 'Service':
            obj_ = KerberosServiceType.factory()
            obj_.build(child_)
            self.set_Service(obj_)
# end class KerberosDelegationType

class KerberosServiceType(GeneratedsSuper):
    """The KerberosServiceType specifies the properties of the Kerberos
    delegation service for the Windows computer account."""

    subclass = None
    superclass = None
    def __init__(self, Computer=None, Name=None, Port=None, User=None):
        self.Computer = Computer
        self.Name = Name
        self.Port = Port
        self.User = User
    def factory(*args_, **kwargs_):
        if KerberosServiceType.subclass:
            return KerberosServiceType.subclass(*args_, **kwargs_)
        else:
            return KerberosServiceType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Computer(self): return self.Computer
    def set_Computer(self, Computer): self.Computer = Computer
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Name(self): return self.Name
    def set_Name(self, Name): self.Name = Name
    def get_Port(self): return self.Port
    def set_Port(self, Port): self.Port = Port
    def get_User(self): return self.User
    def set_User(self, User): self.User = User
    def hasContent_(self):
        if (
            self.Computer is not None or
            self.Name is not None or
            self.Port is not None or
            self.User is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='KerberosServiceType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='KerberosServiceType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinComputerAccountObj:', name_='KerberosServiceType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='KerberosServiceType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Computer is not None:
            self.Computer.export(lwrite, level, 'WinComputerAccountObj:', name_='Computer', pretty_print=pretty_print)
        if self.Name is not None:
            self.Name.export(lwrite, level, 'WinComputerAccountObj:', name_='Name', pretty_print=pretty_print)
        if self.Port is not None:
            self.Port.export(lwrite, level, 'WinComputerAccountObj:', name_='Port', pretty_print=pretty_print)
        if self.User is not None:
            self.User.export(lwrite, level, 'WinComputerAccountObj:', name_='User', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Computer':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Computer(obj_)
        elif nodeName_ == 'Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Name(obj_)
        elif nodeName_ == 'Port':
            obj_ = port_object.PortObjectType.factory()
            obj_.build(child_)
            self.set_Port(obj_)
        elif nodeName_ == 'User':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_User(obj_)
# end class KerberosServiceType

class WindowsComputerAccountObjectType(account_object.AccountObjectType):
    """The WinComputerAccountObject type is intended to characterize
    Windows computer accounts."""

    subclass = None
    superclass = account_object.AccountObjectType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, disabled=None, locked_out=None, Description=None, Domain=None, Fully_Qualified_Name=None, Kerberos=None, Security_ID=None, Security_Type=None, Type=None):
        super(WindowsComputerAccountObjectType, self).__init__(object_reference, Custom_Properties, disabled, locked_out, Description, Domain, )
        self.Fully_Qualified_Name = Fully_Qualified_Name
        self.Kerberos = Kerberos
        self.Security_ID = Security_ID
        self.Security_Type = Security_Type
        self.Type = Type
    def factory(*args_, **kwargs_):
        if WindowsComputerAccountObjectType.subclass:
            return WindowsComputerAccountObjectType.subclass(*args_, **kwargs_)
        else:
            return WindowsComputerAccountObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Fully_Qualified_Name(self): return self.Fully_Qualified_Name
    def set_Fully_Qualified_Name(self, Fully_Qualified_Name): self.Fully_Qualified_Name = Fully_Qualified_Name
    def get_Kerberos(self): return self.Kerberos
    def set_Kerberos(self, Kerberos): self.Kerberos = Kerberos
    def get_Security_ID(self): return self.Security_ID
    def set_Security_ID(self, Security_ID): self.Security_ID = Security_ID
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Security_Type(self): return self.Security_Type
    def set_Security_Type(self, Security_Type): self.Security_Type = Security_Type
    def validate_SIDType(self, value):
        # Validate type cybox_common.SIDType, a restriction on None.
        pass
    def get_Type(self): return self.Type
    def set_Type(self, Type): self.Type = Type
    def hasContent_(self):
        if (
            self.Fully_Qualified_Name is not None or
            self.Kerberos is not None or
            self.Security_ID is not None or
            self.Security_Type is not None or
            self.Type is not None or
            super(WindowsComputerAccountObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='WindowsComputerAccountObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsComputerAccountObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinComputerAccountObj:', name_='WindowsComputerAccountObjectType'):
        super(WindowsComputerAccountObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsComputerAccountObjectType')
    def exportChildren(self, lwrite, level, namespace_='WinComputerAccountObj:', name_='WindowsComputerAccountObjectType', fromsubclass_=False, pretty_print=True):
        super(WindowsComputerAccountObjectType, self).exportChildren(lwrite, level, 'WinComputerAccountObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Fully_Qualified_Name is not None:
            self.Fully_Qualified_Name.export(lwrite, level, 'WinComputerAccountObj:', name_='Fully_Qualified_Name', pretty_print=pretty_print)
        if self.Kerberos is not None:
            self.Kerberos.export(lwrite, level, 'WinComputerAccountObj:', name_='Kerberos', pretty_print=pretty_print)
        if self.Security_ID is not None:
            self.Security_ID.export(lwrite, level, 'WinComputerAccountObj:', name_='Security_ID', pretty_print=pretty_print)
        if self.Security_Type is not None:
            self.Security_Type.export(lwrite, level, 'WinComputerAccountObj:', name_='Security_Type', pretty_print=pretty_print)
        if self.Type is not None:
            self.Type.export(lwrite, level, 'WinComputerAccountObj:', name_='Type', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(WindowsComputerAccountObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Fully_Qualified_Name':
            obj_ = FullyQualifiedNameType.factory()
            obj_.build(child_)
            self.set_Fully_Qualified_Name(obj_)
        elif nodeName_ == 'Kerberos':
            obj_ = KerberosType.factory()
            obj_.build(child_)
            self.set_Kerberos(obj_)
        elif nodeName_ == 'Security_ID':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Security_ID(obj_)
        elif nodeName_ == 'Security_Type':
            obj_ = cybox_common.SIDType.factory()
            obj_.build(child_)
            self.set_Security_Type(obj_)
        elif nodeName_ == 'Type':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Type(obj_)
        super(WindowsComputerAccountObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class WindowsComputerAccountObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'Account': account_object.AccountObjectType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Compiler': cybox_common.CompilerType,
    'Layer4_Protocol': port_object.Layer4ProtocolType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'NetBEUI_Name': cybox_common.StringObjectPropertyType,
    'Tool': cybox_common.ToolInformationType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Bitmask': cybox_common.HexBinaryObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'Domain': cybox_common.StringObjectPropertyType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Security_Type': cybox_common.SIDType,
    'Reference_Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Computer': cybox_common.StringObjectPropertyType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'User': cybox_common.StringObjectPropertyType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Language': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Full_Name': cybox_common.StringObjectPropertyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'Contributors': cybox_common.PersonnelType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Data_Size': cybox_common.DataSizeType,
    'Libraries': cybox_common.LibrariesType,
    'Security_ID': cybox_common.StringObjectPropertyType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StringObjectPropertyType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Ticket': cybox_common.UnsignedLongObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Port_Value': cybox_common.PositiveIntegerObjectPropertyType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Name': cybox_common.StringObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Port': port_object.PortObjectType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Computer_Account'
        rootClass = WindowsComputerAccountObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Computer_Account'
        rootClass = WindowsComputerAccountObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Computer_Account'
        rootClass = WindowsComputerAccountObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Windows_Computer_Account",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "WindowsComputerAccountObjectType",
    "FullyQualifiedNameType",
    "KerberosType",
    "KerberosDelegationType",
    "KerberosServiceType"
    ]
